package org.jfedor.orientationbroadcaster;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.PowerManager;

public class OrientationBroadcaster extends Activity implements SensorEventListener {
    
    SensorManager mSensorManager;
    Sensor mAccelerometer;
    Sensor mMagneticFieldSensor;
    private PowerManager powerManager;
    private PowerManager.WakeLock wakeLock;
    DatagramSocket mSocket;
    InetAddress address;
    float[] accelerometerData = new float[3];
    float[] magneticFieldData = new float[3];
    long counter = 0;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        mAccelerometer = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        mMagneticFieldSensor = mSensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
        
        powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wakeLock = powerManager.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK, "Orientation Broadcaster");
    }

    @Override
    protected void onStart() {
        super.onStart();
        
        mSensorManager.registerListener(this, mAccelerometer, SensorManager.SENSOR_DELAY_FASTEST);
        mSensorManager.registerListener(this, mMagneticFieldSensor, SensorManager.SENSOR_DELAY_FASTEST);
        
        try {
            mSocket = new DatagramSocket();
            address = InetAddress.getByName("255.255.255.255");
        } catch (SocketException e) {
        } catch (UnknownHostException e) {
        }
        
        wakeLock.acquire();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mSensorManager.unregisterListener(this);
        mSocket.close();
        wakeLock.release();
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
        // TODO Auto-generated method stub
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        if (event.sensor == mAccelerometer) {
            System.arraycopy(event.values, 0, accelerometerData, 0, 3);
        } else if (event.sensor == mMagneticFieldSensor) {
            System.arraycopy(event.values, 0, magneticFieldData, 0, 3);
        } else {
            return;
        }
        
        float[] rotationMatrix = new float[9];
        float[] inclinationMatrix = new float[9];
        
        if (SensorManager.getRotationMatrix(rotationMatrix, inclinationMatrix, accelerometerData, magneticFieldData)) {
            ByteBuffer bb = ByteBuffer.allocate(8+4*(3+3+9+9));
            bb.putLong(counter++);
            FloatBuffer fb = bb.asFloatBuffer();
            fb.put(accelerometerData);                
            fb.put(magneticFieldData);
            fb.put(rotationMatrix);
            fb.put(inclinationMatrix);

            byte[] bytes = bb.array();
            DatagramPacket packet = new DatagramPacket(bytes, bytes.length, address, 20123);
            try {
                mSocket.send(packet);
            } catch (IOException e) {
            }
        }
    }
}